@@ -5,6 +5,8 @@ from django.db.models.signals import post_delete
 from django.dispatch.dispatcher import receiver
 from django.contrib.auth.models import User, Group
 from .validators import alphanumeric, numeric
+from django.core.exceptions import ValidationError
+from django.utils.translation import ugettext as _
 
 
 class Organization(models.Model):
@@ -81,7 +83,7 @@ class Part(models.Model):
             'assembly_subpart'))
 
     class Meta():
-        unique_together = ['number_class', 'number_item', 'number_variation']
+        unique_together = ['number_class', 'number_item', 'number_variation', 'organization', ]
 
     def full_part_number(self):
         return "{0}-{1}-{2}".format(self.number_class.code,
@@ -103,6 +105,18 @@ class Part(models.Model):
         used_in_parts = [subpart.assembly_part for subpart in used_in_subparts]
         return used_in_parts
 
+    def where_used_full(self):
+        def where_used_given_part(used_in_parts, part):
+            where_used = part.where_used()
+            used_in_parts.update(where_used)
+            for p in where_used:
+                where_used_given_part(used_in_parts, p)
+            return used_in_parts
+
+        used_in_parts = set()
+        where_used_given_part(used_in_parts, self)
+        return list(used_in_parts)
+
     def files(self):
         partfiles = PartFile.objects.filter(part=self)
         return partfiles
@@ -185,6 +199,13 @@ class Subpart(models.Model):
         Part, related_name='assembly_subpart', null=True)
     count = models.IntegerField(default=1)
 
+    def clean(self):
+        unusable_parts = self.assembly_part.where_used()
+        if self.assembly_subpart in unusable_parts:
+            raise ValidationError(_('Recursive relationship: cannot add a subpart to a part that uses itsself.'), code='invalid')
+        if self.assembly_subpart == self.assembly_part:
+            raise ValidationError(_('Recursive relationship: cannot add a subpart to itsself.'), code='invalid')
+
 
 class Seller(models.Model):
     organization = models.ForeignKey(Organization)
