@@ -2,7 +2,7 @@ import csv
 import codecs
 import logging
 import uuid
-import json
+
 from .settings import MEDIA_URL
 from django.shortcuts import render, get_object_or_404
 from django.http import HttpResponse, HttpResponseRedirect, HttpResponseNotFound
@@ -11,9 +11,14 @@ from django.db import IntegrityError
 from django.db.models import Q
 from django.contrib.auth.decorators import login_required
 from django.core.exceptions import ObjectDoesNotExist
+from django.core.cache import cache
 from django.urls import reverse
+from django.utils.encoding import smart_str
 from django.contrib import messages
+
 from json import loads, dumps
+from math import ceil
+
 from .convert import full_part_number_to_broken_part
 from .models import Part, PartClass, Subpart, SellerPart, Organization, PartFile, Manufacturer
 from .forms import PartInfoForm, PartForm, AddSubpartForm, FileForm, AddSellerPartForm
@@ -53,7 +58,7 @@ def home(request):
         if part.manufacturer is not None and part.manufacturer.name:
             autocomplete_dict.update({ part.manufacturer.name.replace('"', ''): None })
 
-    autocomplete = json.dumps(autocomplete_dict)
+    autocomplete = dumps(autocomplete_dict)
 
     query = request.GET.get('q', '')
     if query:
@@ -98,16 +103,24 @@ def part_info(request, part_id):
         messages.error(request, "Cant access a part that is not yours!")
         return HttpResponseRedirect(reverse('error'))
 
-    part_info_form = PartInfoForm(initial={'quantity': 100})
+    qty_cache_key = part_id + '_qty'
+    qty = cache.get(qty_cache_key, 100)
+    part_info_form = PartInfoForm(initial={'quantity': qty})
     upload_file_to_part_form = FileForm()
 
-    qty = 100
     if request.method == 'POST':
         part_info_form = PartInfoForm(request.POST)
         if part_info_form.is_valid():
             qty = request.POST.get('quantity', 100)
+    
+    cache.set(qty_cache_key, qty, 3600)
+    
+    try:
+        parts = part.indented()
+    except RuntimeError:
+        messages.error(request, "Error: infinite recursion in part relationship. Contact info@indabom.com to resolve.")
+        parts = []
 
-    parts = part.indented()
     extended_cost_complete = True
     unit_cost = 0
     unit_nre = 0
@@ -118,18 +131,21 @@ def part_info(request, part_id):
 
         subpart = item['part']
         seller = subpart.optimal_seller(quantity=extended_quantity)
-        order_qty = extended_quantity if seller is None or extended_quantity > seller.minimum_order_quantity else seller.minimum_order_quantity
+        order_qty = extended_quantity
+        if seller is not None and seller.minimum_order_quantity is not None and extended_quantity > seller.minimum_order_quantity:
+            order_qty = ceil(extended_quantity / float(seller.minimum_order_quantity)) * seller.minimum_order_quantity
 
         item['seller_price'] = seller.unit_cost if seller is not None else 0
         item['seller_nre'] = seller.nre_cost if seller is not None else 0
         item['seller_part'] = seller
+        item['seller_moq'] = seller.minimum_order_quantity if seller is not None else 0
         item['order_quantity'] = order_qty
 
         # then extend that price
         item['extended_cost'] = extended_quantity * \
             seller.unit_cost if seller is not None and seller.unit_cost is not None and extended_quantity is not None else None
         item['out_of_pocket_cost'] = order_qty * \
-            seller.unit_cost if seller is not None and seller.unit_cost is not None else 0
+            float(seller.unit_cost) if seller is not None and seller.unit_cost is not None else 0
 
         unit_cost = (
             unit_cost +
@@ -147,7 +163,7 @@ def part_info(request, part_id):
     # seller_price, seller_nre
 
     extended_cost = unit_cost * int(qty)
-    total_out_of_pocket_cost = unit_out_of_pocket_cost + unit_nre
+    total_out_of_pocket_cost = unit_out_of_pocket_cost + float(unit_nre)
 
     where_used = part.where_used()
     files = part.files()
@@ -168,9 +184,6 @@ def part_export_bom(request, part_id):
     profile = user.bom_profile()
     organization = profile.organization
 
-    response = HttpResponse(content_type='text/csv')
-    response['Content-Disposition'] = 'attachment; filename="indabom_parts_indented.csv"'
-
     try:
         part = Part.objects.get(id=part_id)
     except ObjectDoesNotExist:
@@ -181,8 +194,12 @@ def part_export_bom(request, part_id):
         messages.error(request, "Cant export a part that is not yours!")
         return HttpResponseRedirect(reverse('error'))
 
+    response = HttpResponse(content_type='text/csv')
+    response['Content-Disposition'] = 'attachment; filename="{}_indabom_parts_indented.csv"'.format(part.full_part_number())
+
     bom = part.indented()
-    qty = 1000
+    qty_cache_key = part_id + '_qty'
+    qty = cache.get(qty_cache_key, 1000)
     unit_cost = 0
     unit_out_of_pocket_cost = 0
     unit_nre = 0
@@ -199,8 +216,11 @@ def part_export_bom(request, part_id):
         'part_order_qty',
         'part_seller',
         'part_cost',
+        'part_moq',
         'part_ext_cost',
-        'part_nre']
+        'part_out_of_pocket_cost',
+        'part_nre',
+        'part_lead_time_days', ]
 
     writer = csv.DictWriter(response, fieldnames=fieldnames)
     writer.writeheader()
@@ -210,18 +230,22 @@ def part_export_bom(request, part_id):
 
         subpart = item['part']
         seller = subpart.optimal_seller(quantity=extended_quantity)
-        order_qty = extended_quantity if seller is None or extended_quantity > seller.minimum_order_quantity else seller.minimum_order_quantity
+        order_qty = extended_quantity
+        if seller is not None and seller.minimum_order_quantity is not None and extended_quantity > seller.minimum_order_quantity:
+            order_qty = ceil(extended_quantity / float(seller.minimum_order_quantity)) * seller.minimum_order_quantity
 
         item['seller_price'] = seller.unit_cost if seller is not None else 0
         item['seller_nre'] = seller.nre_cost if seller is not None else 0
         item['seller_part'] = seller
+        item['seller_moq'] = seller.minimum_order_quantity if seller is not None else 0
         item['order_quantity'] = order_qty
+        item['seller_lead_time_days'] = seller.lead_time_days if seller is not None else 0
 
         # then extend that price
         item['extended_cost'] = extended_quantity * \
             seller.unit_cost if seller is not None and seller.unit_cost is not None and extended_quantity is not None else None
         item['out_of_pocket_cost'] = order_qty * \
-            seller.unit_cost if seller is not None and seller.unit_cost is not None else 0
+            float(seller.unit_cost) if seller is not None and seller.unit_cost is not None else 0
 
         unit_cost = (
             unit_cost +
@@ -247,10 +271,13 @@ def part_export_bom(request, part_id):
             'part_order_qty': item['order_quantity'],
             'part_seller': item['seller_part'].seller.name if item['seller_part'] is not None else '',
             'part_cost': item['seller_price'] if item['seller_price'] is not None else 0,
+            'part_moq': item['seller_moq'] if item['seller_moq'] is not None else 0,
             'part_ext_cost': item['extended_cost'] if item['extended_cost'] is not None else 0,
+            'part_out_of_pocket_cost': item['out_of_pocket_cost'],
             'part_nre': item['seller_nre'] if item['seller_nre'] is not None else 0,
+            'part_lead_time_days': item['seller_lead_time_days'], 
         }
-        writer.writerow(row)
+        writer.writerow({k:smart_str(v) for k,v in row.items()})
     return response
 
 
@@ -274,13 +301,13 @@ def part_upload_bom(request, part_id):
                     "utf-8"),
                 delimiter=',',
                 dialect=dialect)
-            headers = reader.next()
+            headers = [h.lower() for h in reader.next()]
             # Subpart.objects.filter(assembly_part=part).delete()
 
             header_error = False
-            if 'part_number' not in headers:
+            if 'part_number' not in headers and 'manufacturer_part_number' not in headers:
                 header_error = True
-                messages.error(request, "Header `part_number` required for upload.")
+                messages.error(request, "Header `part_number` or `manufacturer_part_number` required for upload.")
             if 'quantity' not in headers:
                 header_error = True
                 messages.error(request, "Header `quantity` required for upload.")
@@ -292,17 +319,23 @@ def part_upload_bom(request, part_id):
                 partData = {}
                 for idx, item in enumerate(row):
                     partData[headers[idx]] = item
-                if 'part_number' in partData and 'quantity' in partData:
-                    civ = full_part_number_to_broken_part(
-                        partData['part_number'])
-                    subparts = Part.objects.filter(
-                        number_class=civ['class'],
-                        number_item=civ['item'],
-                        number_variation=civ['variation'])
+                
+                if 'part_number' in partData and 'quantity' in partData and len(partData['part_number']) > 0:
+                    try:
+                        civ = full_part_number_to_broken_part(
+                            partData['part_number'])
+                        subparts = Part.objects.filter(
+                            number_class=civ['class'],
+                            number_item=civ['item'],
+                            number_variation=civ['variation'])
+                    except IndexError:
+                        messages.error(
+                            request, "Invalid part_number: {}".format(partData['part_number']))
+                        continue
 
                     if len(subparts) == 0:
                         messages.info(
-                            request, "Subpart: {} doesn't exist".format(
+                            request, "Subpart: `{}` doesn't exist".format(
                                 partData['part_number']))
                         continue
 
@@ -313,6 +346,28 @@ def part_upload_bom(request, part_id):
                             request, "Recursive part association: a part cant be a subpart of itsself")
                         return HttpResponseRedirect(reverse('part-manage-bom', kwargs={'part_id': part_id}))
 
+                    sp = Subpart(
+                        assembly_part=part,
+                        assembly_subpart=subpart,
+                        count=count)
+                    sp.save()
+                elif 'manufacturer_part_number' in partData and 'quantity' in partData:
+                    mpn = partData['manufacturer_part_number']
+                    subparts = Part.objects.filter(manufacturer_part_number=mpn)
+
+                    if len(subparts) == 0:
+                        messages.info(
+                            request, "Subpart: `{}` doesn't exist".format(
+                                partData['manufacturer_part_number']))
+                        continue
+
+                    subpart = subparts[0]
+                    count = partData['quantity']
+                    if part == subpart:
+                        messages.error(
+                            request, "Recursive part association: a part cant be a subpart of itsself")
+                        return HttpResponseRedirect(reverse('part-manage-bom', kwargs={'part_id': part_id}))
+
                     sp = Subpart(
                         assembly_part=part,
                         assembly_subpart=subpart,
@@ -347,7 +402,7 @@ def upload_parts(request):
                     "utf-8"),
                 delimiter=',',
                 dialect=dialect)
-            headers = reader.next()
+            headers = [h.lower() for h in reader.next()]
             for row in reader:
                 partData = {}
                 for idx, item in enumerate(row):
@@ -371,7 +426,7 @@ def upload_parts(request):
                                 partData['part_class']))
                         return HttpResponseRedirect(reverse('error'))
 
-                    part, created = Part.objects.get_or_create(part_class=part_class,
+                    part, created = Part.objects.get_or_create(number_class=part_class,
                                                                description=partData['description'],
                                                                revision=partData['revision'],
                                                                organization=organization,
@@ -389,10 +444,10 @@ def upload_parts(request):
                     messages.error(
                         request,
                         "File must contain at least the 3 columns (with headers): 'part_class', 'description', and 'revision'.")
-                    return HttpResponseRedirect(reverse('error'))
+                    return TemplateResponse(request, 'bom/upload-parts.html', locals())
         else:
             messages.error(request, "Invalid form input.")
-            return HttpResponseRedirect(reverse('error'))
+            return TemplateResponse(request, 'bom/upload-parts.html', locals())
     else:
         form = FileForm()
         return TemplateResponse(request, 'bom/upload-parts.html', locals())
@@ -433,7 +488,7 @@ def export_part_list(request):
             'part_manufacturer': item.manufacturer.name if item.manufacturer is not None else '',
             'part_manufacturer_part_number': item.manufacturer_part_number if item.manufacturer is not None else '',
         }
-        writer.writerow(row)
+        writer.writerow({k:smart_str(v) for k,v in row.items()})
 
     return response
 
@@ -493,9 +548,9 @@ def part_octopart_match_bom(request, part_id):
             return HttpResponseRedirect(request.META.get('HTTP_REFERER', reverse('home')) + '#sourcing')
 
         if len(seller_parts) > 0:
-            for dp in seller_parts:
+            for sp in seller_parts:
                 try:
-                    dp.save()
+                    sp.save()
                 except IntegrityError:
                     continue
         else:
@@ -517,23 +572,28 @@ def create_part(request):
     if request.method == 'POST':
         form = PartForm(request.POST, organization=organization)
         if form.is_valid():
-            new_part, created = Part.objects.get_or_create(
-                number_class=form.cleaned_data['number_class'],
-                number_item=form.cleaned_data['number_item'],
-                number_variation=form.cleaned_data['number_variation'],
-                manufacturer_part_number=form.cleaned_data['manufacturer_part_number'],
-                manufacturer=form.cleaned_data['manufacturer'],
-                organization=organization,
-                defaults={'description': form.cleaned_data['description'],
-                          'revision': form.cleaned_data['revision'],
-                          }
-            )
+            try:
+                new_part, created = Part.objects.get_or_create(
+                    number_class=form.cleaned_data['number_class'],
+                    number_item=form.cleaned_data['number_item'],
+                    number_variation=form.cleaned_data['number_variation'],
+                    manufacturer_part_number=form.cleaned_data['manufacturer_part_number'],
+                    manufacturer=form.cleaned_data['manufacturer'],
+                    organization=organization,
+                    defaults={'description': form.cleaned_data['description'],
+                            'revision': form.cleaned_data['revision'],
+                            }
+                )
+            except IntegrityError as e:
+                messages.error(request, "Error creating part, please contact info@indabom.com with this information: {}".format(e))
+            
+            if not new_part.manufacturer_part_number:
+                new_part.manufacturer_part_number = new_part.full_part_number()
+                new_part.save()
+            
             return HttpResponseRedirect(
-                reverse(
-                    'part-info',
-                    kwargs={
-                        'part_id': str(
-                            new_part.id)}))
+                reverse('part-info',
+                    kwargs={'part_id': str(new_part.id)}))
     else:
         form = PartForm(organization=organization)
 
@@ -734,14 +794,14 @@ def add_sellerpart(request, part_id):
                 nre_cost=form.cleaned_data['nre_cost'],
                 ncnr=form.cleaned_data['ncnr'],
             )
+        else:
+            messages.error(request,"Form not valid. See error(s) below.")
+            return TemplateResponse(request, 'bom/add-sellerpart.html', locals())
     else:
         if part.organization != organization:
             messages.error(request, "Cant access a part that is not yours!")
             return HttpResponseRedirect(reverse('error'))
-
-        add_sellerpart_form = AddSellerPartForm(
-            organization=organization)
-
+        form = AddSellerPartForm(organization=organization)
         return TemplateResponse(request, 'bom/add-sellerpart.html', locals())
 
     return HttpResponseRedirect(reverse('part-info', kwargs={'part_id': part_id}) + '#sourcing')
